/* Import Geist font from Google Fonts */
@import url('https://fonts.googleapis.com/css2?family=Geist:wght@100..900&display=swap');

/* Override JupyterLab font variables to use Geist */
:root {
  --jp-ui-font-family:
    'Geist', -apple-system, BlinkMacSystemFont, 'Segoe UI', Helvetica, Arial,
    sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol' !important;
  --jp-content-font-family: 'Geist', 'Source Sans Pro', sans-serif !important;
}

/* Ensure Geist is applied to the plugin elements */
.sage-ai-chatbox,
.sage-ai-chat-input,
.sage-ai-rich-chat-input,
.sage-ai-rich-chat-input-wrapper,
.sage-ai-message,
.sage-ai-markdown-content,
.sage-ai-tool-calls,
.sage-ai-step-floating-box,
.sage-ai-banner-header {
  font-family:
    'Geist',
    -apple-system,
    BlinkMacSystemFont,
    'Segoe UI',
    Helvetica,
    Arial,
    sans-serif;
}
